from hvac import utils
from hvac.api.system_backend.system_backend_mixin import SystemBackendMixin


class Quota(SystemBackendMixin):
    def read_quota(self, name):
        """Read quota. Only works when calling on the root namespace.

        Supported methods:
            GET: /sys/quotas/rate-limit/:name. Produces: 200 application/json

        :param name: the name of the quota to look up.
        :type name: str | unicode
        :return: JSON response from API request.
        :rtype: requests.Response
        """
        api_path = utils.format_url(f"/v1/sys/quotas/rate-limit/{name}", name=name)
        return self._adapter.get(url=api_path)

    def list_quotas(self):
        """Retrieve a list of quotas by name. Only works when calling on the root namespace.

        Supported methods:
            LIST: /sys/quotas/rate-limit. Produces: 200 application/json

        :return: JSON response from API request.
        :rtype: requests.Response
        """
        api_path = "/v1/sys/quotas/rate-limit"
        return self._adapter.list(
            url=api_path,
        )

    def create_or_update_quota(
        self,
        name,
        rate,
        path=None,
        interval=None,
        block_interval=None,
        role=None,
        rate_limit_type=None,
        inheritable=None,
    ):
        """Create quota if it doesn't exist or update if already created. Only works when calling on the root namespace.

        Supported methods:
            POST: /sys/quotas/rate-limit. Produces: 204 (empty body)

        :param name: The name of the quota to create or update.
        :type name: str | unicode
        :param path: Path of the mount or namespace to apply the quota.
        :type path: str | unicode
        :param rate: The maximum number of requests in a given interval to be allowed. Must be positive.
        :type rate: float
        :param interval: The duration to enforce rate limit. Default is "1s".
        :type interval: str | unicode
        :param block_interval: If rate limit is reached, how long before client can send requests again.
        :type block_interval: str | unicode
        :param role: If quota is set on an auth mount path, restrict login requests that are made with a specified role.
        :type role: str | unicode
        :param rate_limit_type: Type of rate limit quota. Can be lease-count or rate-limit.
        :type rate_limit_type: str | unicode
        :param inheritable: If set to true on a path that is a namespace, quota will be applied to all child namespaces
        :type inheritable: bool
        :return: API status code from request.
        :rtype: requests.Response
        """
        api_path = utils.format_url("/v1/sys/quotas/rate-limit/{name}", name=name)
        params = utils.remove_nones(
            {
                "name": name,
                "path": path,
                "rate": rate,
                "interval": interval,
                "block_interval": block_interval,
                "role": role,
                "type": rate_limit_type,
                "inheritable": inheritable,
            }
        )
        return self._adapter.post(
            url=api_path,
            json=params,
        )

    def delete_quota(self, name):
        """Delete a given quota. Only works when calling on the root namespace.

        Supported methods:
            DELETE: /sys/quotas/rate-limit. Produces: 204 (empty body)

        :param name: Name of the quota to delete
        :type name: str | unicode
        :return: API status code from request.
        :rtype: requests.Response
        """
        api_path = utils.format_url(f"/v1/sys/quotas/rate-limit/{name}", name=name)
        return self._adapter.delete(
            url=api_path,
        )



#!/usr/bin/env python
"""LDAP methods module."""

from hvac import utils
from hvac.api.vault_api_base import VaultApiBase

DEFAULT_MOUNT_POINT = "ldap"


class Ldap(VaultApiBase):
    """LDAP Secrets Engine (API).
    Reference: https://www.vaultproject.io/api/secret/ldap/index.html
    """

    def configure(
        self,
        binddn=None,
        bindpass=None,
        url=None,
        password_policy=None,
        schema=None,
        userdn=None,
        userattr=None,
        upndomain=None,
        connection_timeout=None,
        request_timeout=None,
        starttls=None,
        insecure_tls=None,
        certificate=None,
        client_tls_cert=None,
        client_tls_key=None,
        mount_point=DEFAULT_MOUNT_POINT,
    ):
        """Configure shared information for the ldap secrets engine.

        Supported methods:
            POST: /{mount_point}/config. Produces: 204 (empty body)

        :param binddn: Distinguished name of object to bind when performing user and group search.
        :type binddn: str | unicode
        :param bindpass: Password to use along with binddn when performing user search.
        :type bindpass: str | unicode
        :param url: Base DN under which to perform user search.
        :type url: str | unicode
        :param userdn: Base DN under which to perform user search.
        :type userdn: str | unicode
        :param upndomain: userPrincipalDomain used to construct the UPN string for the authenticating user.
        :type upndomain: str | unicode
        :param password_policy: The name of the password policy to use to generate passwords.
        :type password_policy: str | unicode
        :param schema: The LDAP schema to use when storing entry passwords. Valid schemas include ``openldap``, ``ad``, and ``racf``.
        :type schema: str | unicode
        :param connection_timeout: Timeout, in seconds, when attempting to connect to the LDAP server before trying the next URL in the configuration.
        :type connection_timeout: int | str
        :param request_timeout: Timeout, in seconds, for the connection when making requests against the server before returning back an error.
        :type request_timeout: int | str
        :param starttls: If true, issues a StartTLS command after establishing an unencrypted connection.
        :type starttls: bool
        :param insecure_tls: If true, skips LDAP server SSL certificate verification - insecure, use with caution!
        :type insecure_tls: bool
        :param certificate: CA certificate to use when verifying LDAP server certificate, must be x509 PEM encoded.
        :type certificate: str | unicode
        :param client_tls_cert: Client certificate to provide to the LDAP server, must be x509 PEM encoded.
        :type client_tls_cert: str | unicode
        :param client_tls_key: Client key to provide to the LDAP server, must be x509 PEM encoded.
        :type client_tls_key: str | unicode
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str | unicode
        :return: The response of the request.
        :rtype: requests.Response
        """
        params = utils.remove_nones(
            {
                "binddn": binddn,
                "bindpass": bindpass,
                "url": url,
                "userdn": userdn,
                "userattr": userattr,
                "upndomain": upndomain,
                "password_policy": password_policy,
                "schema": schema,
                "connection_timeout": connection_timeout,
                "request_timeout": request_timeout,
                "starttls": starttls,
                "insecure_tls": insecure_tls,
                "certificate": certificate,
                "client_tls_cert": client_tls_cert,
                "client_tls_key": client_tls_key,
            }
        )

        api_path = utils.format_url("/v1/{mount_point}/config", mount_point=mount_point)
        return self._adapter.post(
            url=api_path,
            json=params,
        )

    def read_config(self, mount_point=DEFAULT_MOUNT_POINT):
        """Read the configured shared information for the ldap secrets engine.

        Credentials will be omitted from returned data.

        Supported methods:
            GET: /{mount_point}/config. Produces: 200 application/json

        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str | unicode
        :return: The JSON response of the request.
        :rtype: dict
        """
        api_path = utils.format_url("/v1/{mount_point}/config", mount_point=mount_point)
        return self._adapter.get(
            url=api_path,
        )

    def rotate_root(self, mount_point=DEFAULT_MOUNT_POINT):
        """Rotate the root password for the binddn entry used to manage the ldap secrets engine.

        Supported methods:
            POST: /{mount_point}/rotate root. Produces: 200 application/json

        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str | unicode
        :return: The JSON response of the request.
        :rtype: dict
        """
        api_path = utils.format_url(
            "/v1/{mount_point}/rotate-root", mount_point=mount_point
        )
        return self._adapter.post(url=api_path)

    def create_or_update_static_role(
        self,
        name,
        username=None,
        dn=None,
        rotation_period=None,
        mount_point=DEFAULT_MOUNT_POINT,
    ):
        """This endpoint creates or updates the ldap static role definition.

        :param name: Specifies the name of an existing static role against which to create this ldap credential.
        :type name: str | unicode
        :param username: The name of a pre-existing service account in LDAP that maps to this static role.
            This value is required on create and cannot be updated.
        :type username: str | unicode
        :param dn: Distinguished name of the existing LDAP entry to manage password rotation for (takes precedence over username).
            Optional but cannot be modified after creation.
        :type dn: str | unicode
        :param rotation_period: How often Vault should rotate the password.
            This is provided as a string duration with a time suffix like "30s" or "1h" or as seconds.
            If not provided, the default Vault rotation_period is used.
        :type rotation_period: str | unicode
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str | unicode
        :return: The response of the request.
        :rtype: requests.Response
        """
        api_path = utils.format_url("/v1/{}/static-role/{}", mount_point, name)
        params = {"username": username, "rotation_period": rotation_period}
        params.update(utils.remove_nones({"dn": dn}))
        return self._adapter.post(
            url=api_path,
            json=params,
        )

    def read_static_role(self, name, mount_point=DEFAULT_MOUNT_POINT):
        """This endpoint queries for information about an ldap static role with the given name.
        If no role exists with that name, a 404 is returned.
        :param name: Specifies the name of the static role to query.
        :type name: str | unicode
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str | unicode
        :return: The response of the request.
        :rtype: requests.Response
        """
        api_path = utils.format_url("/v1/{}/static-role/{}", mount_point, name)
        return self._adapter.get(
            url=api_path,
        )

    def list_static_roles(self, mount_point=DEFAULT_MOUNT_POINT):
        """This endpoint lists all existing static roles in the secrets engine.
        :return: The response of the request.
        :rtype: requests.Response
        """
        api_path = utils.format_url("/v1/{}/static-role", mount_point)
        return self._adapter.list(
            url=api_path,
        )

    def delete_static_role(self, name, mount_point=DEFAULT_MOUNT_POINT):
        """This endpoint deletes an ldap static role with the given name.
        Even if the role does not exist, this endpoint will still return a successful response.
        :param name: Specifies the name of the role to delete.
        :type name: str | unicode
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str | unicode
        :return: The response of the request.
        :rtype: requests.Response
        """
        api_path = utils.format_url("/v1/{}/static-role/{}", mount_point, name)
        return self._adapter.delete(
            url=api_path,
        )

    def generate_static_credentials(self, name, mount_point=DEFAULT_MOUNT_POINT):
        """This endpoint retrieves the previous and current LDAP password for
        the associated account (or rotate if required)

        :param name: Specifies the name of the static role to request credentials from.
        :type name: str | unicode
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str | unicode
        :return: The response of the request.
        :rtype: requests.Response
        """
        api_path = utils.format_url("/v1/{}/static-cred/{}", mount_point, name)
        return self._adapter.get(
            url=api_path,
        )

    def rotate_static_credentials(self, name, mount_point=DEFAULT_MOUNT_POINT):
        """This endpoint rotates the password of an existing static role.

        :param name: Specifies the name of the static role to rotate credentials for.
        :type name: str | unicode
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str | unicode
        :return: The response of the request.
        :rtype: requests.Response
        """
        api_path = utils.format_url("/v1/{}/rotate-role/{}", mount_point, name)
        return self._adapter.post(
            url=api_path,
        )



#!/usr/bin/env python
"""Constants related to the hvac.Client class."""

from os import getenv

DEPRECATED_PROPERTIES = {}
# ^ this follows the format defined in utils.getattr_with_deprecated_properties
# example:
#   {
#       "old_property_one": {
#           "to_be_removed_in_version": "99.0.0",
#           "client_property": "auth",
#       },
#       "old_property_two": {
#           "to_be_removed_in_version": "99.0.0",
#           "client_property": "secrets",
#           "new_property": "new_property_two",
#       },
#   }
#
# Result is that `client.old_property_one` will return the value of `client.auth.old_property_one`,
# and `client.old_property_two` will return `client.secrets.new_property_two`.

DEFAULT_URL = "http://localhost:8200"
VAULT_CACERT = getenv("VAULT_CACERT")
VAULT_CAPATH = getenv("VAULT_CAPATH")
VAULT_CLIENT_CERT = getenv("VAULT_CLIENT_CERT")
VAULT_CLIENT_KEY = getenv("VAULT_CLIENT_KEY")



import warnings
from hvac.api.system_backend.system_backend_mixin import SystemBackendMixin
from hvac.exceptions import ParamValidationError


class Init(SystemBackendMixin):
    def read_init_status(self):
        """Read the initialization status of Vault.

        Supported methods:
            GET: /sys/init. Produces: 200 application/json

        :return: The JSON response of the request.
        :rtype: dict
        """
        api_path = "/v1/sys/init"
        return self._adapter.get(
            url=api_path,
        )

    def is_initialized(self):
        """Determine is Vault is initialized or not.

        :return: True if Vault is initialized, False otherwise.
        :rtype: bool
        """
        status = self.read_init_status()
        return status["initialized"]

    def initialize(
        self,
        secret_shares=None,
        secret_threshold=None,
        pgp_keys=None,
        root_token_pgp_key=None,
        stored_shares=None,
        recovery_shares=None,
        recovery_threshold=None,
        recovery_pgp_keys=None,
    ):
        """Initialize a new Vault.

        The Vault must not have been previously initialized. The recovery options, as well as the stored shares option,
        are only available when using Vault HSM.

        Supported methods:
            PUT: /sys/init. Produces: 200 application/json

        :param secret_shares: The number of shares to split the master key into.
        :type secret_shares: int
        :param secret_threshold: Specifies the number of shares required to reconstruct the master key. This must be
            less than or equal secret_shares. If using Vault HSM with auto-unsealing, this value must be the same as
            secret_shares, or omitted, depending on the version of Vault and the seal type.
        :type secret_threshold: int
        :param pgp_keys: List of PGP public keys used to encrypt the output unseal keys.
            Ordering is preserved. The keys must be base64-encoded from their original binary representation.
            The size of this array must be the same as secret_shares.
        :type pgp_keys: list
        :param root_token_pgp_key: Specifies a PGP public key used to encrypt the initial root token. The
            key must be base64-encoded from its original binary representation.
        :type root_token_pgp_key: str | unicode
        :param stored_shares: <enterprise only> Specifies the number of shares that should be encrypted by the HSM and
            stored for auto-unsealing. Currently must be the same as secret_shares.
        :type stored_shares: int
        :param recovery_shares: <enterprise only> Specifies the number of shares to split the recovery key into.
        :type recovery_shares: int
        :param recovery_threshold: <enterprise only> Specifies the number of shares required to reconstruct the recovery
            key. This must be less than or equal to recovery_shares.
        :type recovery_threshold: int
        :param recovery_pgp_keys: <enterprise only> Specifies an array of PGP public keys used to encrypt the output
            recovery keys. Ordering is preserved. The keys must be base64-encoded from their original binary
            representation. The size of this array must be the same as recovery_shares.
        :type recovery_pgp_keys: list
        :return: The JSON response of the request.
        :rtype: dict
        """

        # TODO(v3.0.0): remove this
        if recovery_shares is None and secret_shares is None:
            msg = (
                "The secret_shares parameter will default to None in hvac v3.0.0. "
                "To use the old default with no warning, explicitly set this value to 5. "
                "See https://github.com/hvac/hvac/issues/1030"
            )
            warnings.warn(
                message=msg,
                category=DeprecationWarning,
                stacklevel=2,
            )
            secret_shares = 5

        # TODO(v3.0.0): remove this
        if recovery_threshold is None and secret_threshold is None:
            msg = (
                "The secret_threshold parameter will default to None in hvac v3.0.0. "
                "To use the old default with no warning, explicitly set this value to 3. "
                "See https://github.com/hvac/hvac/issues/1030"
            )
            warnings.warn(
                message=msg,
                category=DeprecationWarning,
                stacklevel=2,
            )
            secret_threshold = 3

        params = {
            "secret_shares": secret_shares,
            "secret_threshold": secret_threshold,
            "root_token_pgp_key": root_token_pgp_key,
        }

        if pgp_keys is not None and secret_shares is not None:
            if len(pgp_keys) != secret_shares:
                raise ParamValidationError(
                    "length of pgp_keys list argument must equal secret_shares value"
                )
            params["pgp_keys"] = pgp_keys

        if stored_shares is not None and secret_shares is not None:
            if stored_shares != secret_shares:
                raise ParamValidationError(
                    "value for stored_shares argument must equal secret_shares argument"
                )
            params["stored_shares"] = stored_shares

        if recovery_shares is not None:
            params["recovery_shares"] = recovery_shares

            if recovery_threshold is not None:
                if recovery_threshold > recovery_shares:
                    error_msg = "value for recovery_threshold argument must be less than or equal to recovery_shares argument"
                    raise ParamValidationError(error_msg)
                params["recovery_threshold"] = recovery_threshold

            if recovery_pgp_keys is not None:
                if len(recovery_pgp_keys) != recovery_shares:
                    raise ParamValidationError(
                        "length of recovery_pgp_keys list argument must equal recovery_shares value"
                    )
                params["recovery_pgp_keys"] = recovery_pgp_keys

        api_path = "/v1/sys/init"
        return self._adapter.put(
            url=api_path,
            json=params,
        )
